"use client";
import { useEffect, useState } from 'react'

export const useLocalStorageString = (
	key: string,
	initialValue: string
): [string, (value: string) => void] => {
	const [storedValue, setStoredValue] = useState(initialValue)

	useEffect(() => {
		// Retrieve from localStorage
		if (typeof window !== 'undefined' && window.localStorage) {
			const item = window.localStorage.getItem(key)
			if (item) {
				setStoredValue(item)
			}
		}
	}, [key])

	const setValue = (value: string) => {
		if (typeof window !== 'undefined' && window.localStorage) {
			// Save state
			setStoredValue(value)
			// Save to localStorage

			window.localStorage.setItem(key, value)
		}
	}
	return [storedValue, setValue]
}

export const useLocalStorage = <T>(
	key: string,
	initialValue: T
): [T, (value: T) => void] => {
	const [storedValue, setStoredValue] = useState(initialValue)

	useEffect(() => {
		// Retrieve from localStorage
		if (typeof window !== 'undefined' && window.localStorage) {
			const item = window.localStorage.getItem(key)
			if (item) {
				setStoredValue(JSON.parse(item))
			}
		}
	}, [key])

	const setValue = (value: T) => {
		// Save state
		if (typeof window !== 'undefined' && window.localStorage) {
			setStoredValue(value)
			// Save to localStorage
			window.localStorage.setItem(key, JSON.stringify(value))
		}
	}
	return [storedValue, setValue]
}
