import { AIAPIPrefix, SearchShareAPIPrefix } from "@/const/api";
import { Relate } from "@/types/relate";
import { Source } from "@/types/source";
import { fetchStream } from "@/utils/fetch-stream";

const LLM_SPLIT = "__LLM_RESPONSE__";
const RELATED_SPLIT = "__RELATED_QUESTIONS__";

export const parseStreaming = async (
  controller: AbortController,
  query: string,
  search_uuid: string,
  mode: "query" | "share",
  onSources: (value: Source[]) => void,
  onMarkdown: (value: string) => void,
  onRelates: (value: Relate[]) => void,
  onServerQuery: (value: string) => void,
  onError?: (status: number) => void,
) => {
  const decoder = new TextDecoder();
  let uint8Array = new Uint8Array();
  let chunks = "";
  let sourcesEmitted = false;
  const response = await fetch(mode === "query" ? `${AIAPIPrefix}/v1/chat/search` : `${SearchShareAPIPrefix}/v1/chat/share`, {
    method: "POST",
    headers: {
      "Content-Type": "application/json",
      Accept: "*./*",
    },
    signal: controller.signal,
    body: JSON.stringify({
      query,
      search_uuid,
    }),
  });
  if (response.status !== 200) {
    onError?.(response.status);
    return;
  } else {
    onError?.(200);
  }
  const markdownParse = (text: string) => {
    onMarkdown(
      text
        .replace(/\[\[([cC])itation/g, "[citation")
        .replace(/[cC]itation:(\d+)]]/g, "citation:$1]")
        .replace(/\[\[([cC]itation:\d+)]](?!])/g, `[$1]`)
        .replace(/\[[cC]itation:(\d+)]/g, "[citation]($1)")
        .replace(/\[(\d+)\]/g, "[citation]($1)")
        .replace(/\[引用:(\d+)\]/g, "[citation]($1)")
        .replace(/\[引用(\d+)\]/g, "[citation]($1)"),
    );
  };
  fetchStream(
    response,
    (chunk) => {
      uint8Array = new Uint8Array([...uint8Array, ...chunk]);
      chunks = decoder.decode(uint8Array, { stream: true });
      if (chunks.includes(LLM_SPLIT)) {
        const [sources, rest] = chunks.split(LLM_SPLIT);
        if (!sourcesEmitted) {
          try {
            onServerQuery((JSON.parse(sources)).query);
            onSources((JSON.parse(sources)).contexts);
            onError?.(200);
          } catch (e) {
            try {
              const shortsource = sources.slice(1)
              onServerQuery((JSON.parse(shortsource)).query);
              onSources((JSON.parse(shortsource)).contexts);
              onError?.(200);
            } catch (e) {
              console.error("parse sources error", e);
            }
          }
        }
        sourcesEmitted = true;
        if (rest.includes(RELATED_SPLIT)) {
          const [md] = rest.split(RELATED_SPLIT);
          markdownParse(md);
          onError?.(200);
        } else {
          markdownParse(rest);
          onError?.(200);
        }
      }
    },
    () => {
      const [_, relates] = chunks.split(RELATED_SPLIT);
      try {
        onRelates(JSON.parse(relates));
        onError?.(200);
        window.history.replaceState(null, "", "/search?rid=" + search_uuid);
      } catch (e) {
        console.error("parse relates error", e);
      }
    },
  );
};
